@@ -33,6 +33,7 @@
 #include "gdl-dock-paned.h"
 #include "gdl-dock-notebook.h"
 #include "gdl-dock-placeholder.h"
+#include "gdl-preview-window.h"
 
 #include "libgdlmarshal.h"
 
@@ -61,10 +62,15 @@ static void  gdl_dock_notify_cb       (GObject      *object,
 
 static void  gdl_dock_set_title       (GdlDock      *dock);
 
-static void  gdl_dock_destroy         (GtkObject    *object);
+static void  gdl_dock_dispose         (GObject      *object);
+
+static void  gdl_dock_get_preferred_width  (GtkWidget *widget,
+                                            gint      *minimum,
+                                            gint      *natural);
+static void  gdl_dock_get_preferred_height (GtkWidget *widget,
+                                            gint      *minimum,
+                                            gint      *natural);
 
-static void  gdl_dock_size_request    (GtkWidget      *widget,
-                                       GtkRequisition *requisition);
 static void  gdl_dock_size_allocate   (GtkWidget      *widget,
                                        GtkAllocation  *allocation);
 static void  gdl_dock_map             (GtkWidget      *widget);
@@ -122,8 +128,7 @@ struct _GdlDockPrivate
     gint                width;
     gint                height;
 
-    /* auxiliary fields */
-    GdkGC              *xor_gc;
+    GtkWidget          *area_window;
 };
 
 enum {
@@ -154,13 +159,11 @@ static void
 gdl_dock_class_init (GdlDockClass *klass)
 {
     GObjectClass       *g_object_class;
-    GtkObjectClass     *gtk_object_class;
     GtkWidgetClass     *widget_class;
     GtkContainerClass  *container_class;
     GdlDockObjectClass *object_class;
     
     g_object_class = G_OBJECT_CLASS (klass);
-    gtk_object_class = GTK_OBJECT_CLASS (klass);
     widget_class = GTK_WIDGET_CLASS (klass);
     container_class = GTK_CONTAINER_CLASS (klass);
     object_class = GDL_DOCK_OBJECT_CLASS (klass);
@@ -168,6 +171,7 @@ gdl_dock_class_init (GdlDockClass *klass)
     g_object_class->constructor = gdl_dock_constructor;
     g_object_class->set_property = gdl_dock_set_property;
     g_object_class->get_property = gdl_dock_get_property;
+    g_object_class->dispose = gdl_dock_dispose;
     
     /* properties */
 
@@ -217,10 +221,9 @@ gdl_dock_class_init (GdlDockClass *klass)
                           G_MININT, G_MAXINT, 0,
                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
                           GDL_DOCK_PARAM_EXPORT));
-    
-    gtk_object_class->destroy = gdl_dock_destroy;
 
-    widget_class->size_request = gdl_dock_size_request;
+    widget_class->get_preferred_width = gdl_dock_get_preferred_width;
+    widget_class->get_preferred_height = gdl_dock_get_preferred_height;
     widget_class->size_allocate = gdl_dock_size_allocate;
     widget_class->map = gdl_dock_map;
     widget_class->unmap = gdl_dock_unmap;
@@ -231,7 +234,8 @@ gdl_dock_class_init (GdlDockClass *klass)
     container_class->remove = gdl_dock_remove;
     container_class->forall = gdl_dock_forall;
     container_class->child_type = gdl_dock_child_type;
-    
+    gtk_container_class_handle_border_width (container_class);
+
     object_class->is_compound = TRUE;
     
     object_class->detach = gdl_dock_detach;
@@ -256,17 +260,22 @@ gdl_dock_class_init (GdlDockClass *klass)
                       0);
 
     klass->layout_changed = NULL;
+
+    g_type_class_add_private (object_class, sizeof (GdlDockPrivate));
 }
 
 static void
 gdl_dock_init (GdlDock *dock)
 {
+    dock->priv = G_TYPE_INSTANCE_GET_PRIVATE (dock,
+                                              GDL_TYPE_DOCK,
+                                              GdlDockPrivate);
+
     gtk_widget_set_has_window (GTK_WIDGET (dock), FALSE);
 
     dock->root = NULL;
-    dock->_priv = g_new0 (GdlDockPrivate, 1);
-    dock->_priv->width = -1;
-    dock->_priv->height = -1;
+    dock->priv->width = -1;
+    dock->priv->height = -1;
 }
 
 static gboolean 
@@ -279,10 +288,10 @@ gdl_dock_floating_configure_event_cb (GtkWidget         *widget,
     g_return_val_if_fail (user_data != NULL && GDL_IS_DOCK (user_data), TRUE);
 
     dock = GDL_DOCK (user_data);
-    dock->_priv->float_x = event->x;
-    dock->_priv->float_y = event->y;
-    dock->_priv->width = event->width;
-    dock->_priv->height = event->height;
+    dock->priv->float_x = event->x;
+    dock->priv->float_y = event->y;
+    dock->priv->width = event->width;
+    dock->priv->height = event->height;
 
     return FALSE;
 }
@@ -310,29 +319,29 @@ gdl_dock_constructor (GType                  type,
             gdl_dock_object_bind (GDL_DOCK_OBJECT (dock), G_OBJECT (master));
         }
 
-        if (dock->_priv->floating) {
+        if (dock->priv->floating) {
             GdlDockObject *controller;
 
             /* create floating window for this dock */
-            dock->_priv->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-            g_object_set_data (G_OBJECT (dock->_priv->window), "dock", dock);
+            dock->priv->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+            g_object_set_data (G_OBJECT (dock->priv->window), "dock", dock);
 
             /* set position and default size */
-            gtk_window_set_position (GTK_WINDOW (dock->_priv->window),
+            gtk_window_set_position (GTK_WINDOW (dock->priv->window),
                                      GTK_WIN_POS_MOUSE);
-            gtk_window_set_default_size (GTK_WINDOW (dock->_priv->window),
-                                         dock->_priv->width,
-                                         dock->_priv->height);
-            gtk_window_set_type_hint (GTK_WINDOW (dock->_priv->window),
+            gtk_window_set_default_size (GTK_WINDOW (dock->priv->window),
+                                         dock->priv->width,
+                                         dock->priv->height);
+            gtk_window_set_type_hint (GTK_WINDOW (dock->priv->window),
                                       GDK_WINDOW_TYPE_HINT_NORMAL);
 
             /* metacity ignores this */
-            gtk_window_move (GTK_WINDOW (dock->_priv->window),
-                             dock->_priv->float_x,
-                             dock->_priv->float_y);
+            gtk_window_move (GTK_WINDOW (dock->priv->window),
+                             dock->priv->float_x,
+                             dock->priv->float_y);
 
             /* connect to the configure event so we can track down window geometry */
-            g_signal_connect (dock->_priv->window, "configure_event",
+            g_signal_connect (dock->priv->window, "configure_event",
                               (GCallback) gdl_dock_floating_configure_event_cb,
                               dock);
 
@@ -342,9 +351,9 @@ gdl_dock_constructor (GType                  type,
             g_signal_connect (dock, "notify::long-name",
                               (GCallback) gdl_dock_notify_cb, NULL);
 
-            gtk_container_add (GTK_CONTAINER (dock->_priv->window), GTK_WIDGET (dock));
+            gtk_container_add (GTK_CONTAINER (dock->priv->window), GTK_WIDGET (dock));
 
-            g_signal_connect (dock->_priv->window, "delete_event",
+            g_signal_connect (dock->priv->window, "delete_event",
                               G_CALLBACK (gdl_dock_floating_window_delete_event_cb), 
                               NULL);
         }
@@ -364,7 +373,7 @@ gdl_dock_set_property  (GObject      *object,
     
     switch (prop_id) {
         case PROP_FLOATING:
-            dock->_priv->floating = g_value_get_boolean (value);
+            dock->priv->floating = g_value_get_boolean (value);
             break;
         case PROP_DEFAULT_TITLE:
             if (GDL_DOCK_OBJECT (object)->master)
@@ -373,16 +382,16 @@ gdl_dock_set_property  (GObject      *object,
                               NULL);
             break;
         case PROP_WIDTH:
-            dock->_priv->width = g_value_get_int (value);
+            dock->priv->width = g_value_get_int (value);
             break;
         case PROP_HEIGHT:
-            dock->_priv->height = g_value_get_int (value);
+            dock->priv->height = g_value_get_int (value);
             break;
         case PROP_FLOAT_X:
-            dock->_priv->float_x = g_value_get_int (value);
+            dock->priv->float_x = g_value_get_int (value);
             break;
         case PROP_FLOAT_Y:
-            dock->_priv->float_y = g_value_get_int (value);
+            dock->priv->float_y = g_value_get_int (value);
             break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -394,10 +403,10 @@ gdl_dock_set_property  (GObject      *object,
         case PROP_HEIGHT:
         case PROP_FLOAT_X:
         case PROP_FLOAT_Y:
-            if (dock->_priv->floating && dock->_priv->window) {
-                gtk_window_resize (GTK_WINDOW (dock->_priv->window),
-                                   dock->_priv->width,
-                                   dock->_priv->height);
+            if (dock->priv->floating && dock->priv->window) {
+                gtk_window_resize (GTK_WINDOW (dock->priv->window),
+                                   dock->priv->width,
+                                   dock->priv->height);
             }
             break;
     }
@@ -413,7 +422,7 @@ gdl_dock_get_property  (GObject      *object,
 
     switch (prop_id) {
         case PROP_FLOATING:
-            g_value_set_boolean (value, dock->_priv->floating);
+            g_value_set_boolean (value, dock->priv->floating);
             break;
         case PROP_DEFAULT_TITLE:
             if (GDL_DOCK_OBJECT (object)->master) {
@@ -421,26 +430,23 @@ gdl_dock_get_property  (GObject      *object,
                 g_object_get (GDL_DOCK_OBJECT (object)->master,
                               "default-title", &default_title,
                               NULL);
-#if GLIB_CHECK_VERSION(2,3,0)
+
                 g_value_take_string (value, default_title);
-#else
-                g_value_set_string_take_ownership (value, default_title);
-#endif
             }
             else
                 g_value_set_string (value, NULL);
             break;
         case PROP_WIDTH:
-            g_value_set_int (value, dock->_priv->width);
+            g_value_set_int (value, dock->priv->width);
             break;
         case PROP_HEIGHT:
-            g_value_set_int (value, dock->_priv->height);
+            g_value_set_int (value, dock->priv->height);
             break;
         case PROP_FLOAT_X:
-            g_value_set_int (value, dock->_priv->float_x);
+            g_value_set_int (value, dock->priv->float_x);
             break;
         case PROP_FLOAT_Y:
-            g_value_set_int (value, dock->_priv->float_y);
+            g_value_set_int (value, dock->priv->float_y);
             break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -454,10 +460,10 @@ gdl_dock_set_title (GdlDock *dock)
     GdlDockObject *object = GDL_DOCK_OBJECT (dock);
     gchar         *title = NULL;
     
-    if (!dock->_priv->window)
+    if (!dock->priv->window)
         return;
     
-    if (!dock->_priv->auto_title && object->long_name) {
+    if (!dock->priv->auto_title && object->long_name) {
         title = object->long_name;
     }
     else if (object->master) {
@@ -470,12 +476,12 @@ gdl_dock_set_title (GdlDock *dock)
     
     if (!title) {
         /* set a default title in the long_name */
-        dock->_priv->auto_title = TRUE;
+        dock->priv->auto_title = TRUE;
         title = g_strdup_printf (
             _("Dock #%d"), GDL_DOCK_MASTER (object->master)->dock_number++);
     }
 
-    gtk_window_set_title (GTK_WINDOW (dock->_priv->window), title);
+    gtk_window_set_title (GTK_WINDOW (dock->priv->window), title);
 
     g_free (title);
 }
@@ -495,66 +501,74 @@ gdl_dock_notify_cb (GObject    *object,
     if (long_name)
     {
         dock = GDL_DOCK (object);
-        dock->_priv->auto_title = FALSE;
+        dock->priv->auto_title = FALSE;
         gdl_dock_set_title (dock);
     }
     g_free (long_name);
 }
 
 static void
-gdl_dock_destroy (GtkObject *object)
+gdl_dock_dispose (GObject *object)
 {
-    GdlDock *dock = GDL_DOCK (object);
+    GdlDockPrivate *priv = GDL_DOCK (object)->priv;
 
-    if (dock->_priv) {
-        GdlDockPrivate *priv = dock->_priv;
-        dock->_priv = NULL;
-
-        if (priv->window) {
-            gtk_widget_destroy (priv->window);
-            priv->floating = FALSE;
-            priv->window = NULL;
-        }
-        
-        /* destroy the xor gc */
-        if (priv->xor_gc) {
-            g_object_unref (priv->xor_gc);
-            priv->xor_gc = NULL;
-        }
-
-        g_free (priv);
+    if (priv->window) {
+        gtk_widget_destroy (priv->window);
+        priv->floating = FALSE;
+        priv->window = NULL;
     }
-    
-   GTK_OBJECT_CLASS (gdl_dock_parent_class)->destroy (object);
+
+    if (priv->area_window)
+    {
+        gtk_widget_destroy (priv->area_window);
+        priv->area_window = NULL;
+    }
+
+    G_OBJECT_CLASS (gdl_dock_parent_class)->dispose (object);
 }
 
 static void
-gdl_dock_size_request (GtkWidget      *widget,
-                       GtkRequisition *requisition)
+gdl_dock_get_size (GtkWidget     *widget,
+                   GtkOrientation orientation,
+                   gint          *minimum,
+                   gint          *natural)
 {
     GdlDock       *dock;
     GtkContainer  *container;
-    guint          border_width;
 
     g_return_if_fail (widget != NULL);
     g_return_if_fail (GDL_IS_DOCK (widget));
 
     dock = GDL_DOCK (widget);
     container = GTK_CONTAINER (widget);
-    border_width = gtk_container_get_border_width (container);
+
+    *minimum = *natural = 0;
 
     /* make request to root */
-    if (dock->root && gtk_widget_get_visible (GTK_WIDGET (dock->root)))
-        gtk_widget_size_request (GTK_WIDGET (dock->root), requisition);
-    else {
-        requisition->width = 0;
-        requisition->height = 0;
-    };
+    if (dock->root && gtk_widget_get_visible (GTK_WIDGET (dock->root))) {
+        if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+            gtk_widget_get_preferred_width (GTK_WIDGET (dock->root), minimum, natural);
+        }
+        else {
+            gtk_widget_get_preferred_height (GTK_WIDGET (dock->root), minimum, natural);
+        }
+    }
+}
 
-    requisition->width += 2 * border_width;
-    requisition->height += 2 * border_width;
+static void
+gdl_dock_get_preferred_width (GtkWidget *widget,
+                              gint      *minimum,
+                              gint      *natural)
+{
+    gdl_dock_get_size (widget, GTK_ORIENTATION_HORIZONTAL, minimum, natural);
+}
 
-    //gtk_widget_size_request (widget, requisition);    
+static void
+gdl_dock_get_preferred_height (GtkWidget *widget,
+                               gint      *minimum,
+                               gint      *natural)
+{
+    gdl_dock_get_size (widget, GTK_ORIENTATION_VERTICAL, minimum, natural);
 }
 
 static void
@@ -563,23 +577,15 @@ gdl_dock_size_allocate (GtkWidget     *widget,
 {
     GdlDock       *dock;
     GtkContainer  *container;
-    guint          border_width;
 
     g_return_if_fail (widget != NULL);
     g_return_if_fail (GDL_IS_DOCK (widget));
-    
+
     dock = GDL_DOCK (widget);
     container = GTK_CONTAINER (widget);
-    border_width = gtk_container_get_border_width (container);
 
     gtk_widget_set_allocation (widget, allocation);
 
-    /* reduce allocation by border width */
-    allocation->x += border_width;
-    allocation->y += border_width;
-    allocation->width = MAX (1, allocation->width - 2 * border_width);
-    allocation->height = MAX (1, allocation->height - 2 * border_width);
-
     if (dock->root && gtk_widget_get_visible (GTK_WIDGET (dock->root)))
         gtk_widget_size_allocate (GTK_WIDGET (dock->root), allocation);
 }
@@ -623,8 +629,8 @@ gdl_dock_unmap (GtkWidget *widget)
             gtk_widget_unmap (child);
     }
     
-    if (dock->_priv->window)
-        gtk_widget_unmap (dock->_priv->window);
+    if (dock->priv->window)
+        gtk_widget_unmap (dock->priv->window);
 }
 
 static void
@@ -648,8 +654,8 @@ gdl_dock_show (GtkWidget *widget)
     GTK_WIDGET_CLASS (gdl_dock_parent_class)->show (widget);
     
     dock = GDL_DOCK (widget);
-    if (dock->_priv->floating && dock->_priv->window)
-        gtk_widget_show (dock->_priv->window);
+    if (dock->priv->floating && dock->priv->window)
+        gtk_widget_show (dock->priv->window);
 
     if (GDL_DOCK_IS_CONTROLLER (dock)) {
         gdl_dock_master_foreach_toplevel (GDL_DOCK_OBJECT_GET_MASTER (dock),
@@ -669,8 +675,8 @@ gdl_dock_hide (GtkWidget *widget)
     GTK_WIDGET_CLASS (gdl_dock_parent_class)->hide (widget);
     
     dock = GDL_DOCK (widget);
-    if (dock->_priv->floating && dock->_priv->window)
-        gtk_widget_hide (dock->_priv->window);
+    if (dock->priv->floating && dock->priv->window)
+        gtk_widget_hide (dock->priv->window);
 
     if (GDL_DOCK_IS_CONTROLLER (dock)) {
         gdl_dock_master_foreach_toplevel (GDL_DOCK_OBJECT_GET_MASTER (dock),
@@ -766,7 +772,7 @@ gdl_dock_reduce (GdlDockObject *object)
 
     } else if (!GDL_DOCK_OBJECT_ATTACHED (dock)) {
         /* if the user explicitly detached the object */
-        if (dock->_priv->floating)
+        if (dock->priv->floating)
             gtk_widget_hide (GTK_WIDGET (dock));
         else {
             GtkWidget *widget = GTK_WIDGET (object);
@@ -965,7 +971,7 @@ gdl_dock_reorder (GdlDockObject    *object,
     GdlDock *dock = GDL_DOCK (object);
     gboolean handled = FALSE;
     
-    if (dock->_priv->floating &&
+    if (dock->priv->floating &&
         new_position == GDL_DOCK_FLOATING &&
         dock->root == requestor) {
         
@@ -973,7 +979,7 @@ gdl_dock_reorder (GdlDockObject    *object,
             GdkRectangle *rect;
 
             rect = g_value_get_boxed (other_data);
-            gtk_window_move (GTK_WINDOW (dock->_priv->window),
+            gtk_window_move (GTK_WINDOW (dock->priv->window),
                              rect->x,
                              rect->y);
             handled = TRUE;
@@ -1008,8 +1014,8 @@ gdl_dock_present (GdlDockObject *object,
 {
     GdlDock *dock = GDL_DOCK (object);
 
-    if (dock->_priv->floating)
-        gtk_window_present (GTK_WINDOW (dock->_priv->window));
+    if (dock->priv->floating)
+        gtk_window_present (GTK_WINDOW (dock->priv->window));
 }
 
 
@@ -1321,45 +1327,25 @@ gdl_dock_object_get_toplevel (GdlDockObject *object)
 
 void
 gdl_dock_xor_rect (GdlDock      *dock,
-                   GdkRectangle *rect)
+                   cairo_rectangle_int_t *rect)
 {
-    GtkWidget *widget;
-    GdkWindow *window;
-    gint8      dash_list [2];
-
-    widget = GTK_WIDGET (dock);
-
-    if (!dock->_priv->xor_gc) {
-        if (gtk_widget_get_realized (widget)) {
-            GdkGCValues values;
-
-            values.function = GDK_INVERT;
-            values.subwindow_mode = GDK_INCLUDE_INFERIORS;
-            dock->_priv->xor_gc = gdk_gc_new_with_values 
-                (gtk_widget_get_window (widget), &values, GDK_GC_FUNCTION | GDK_GC_SUBWINDOW);
-        } else 
-            return;
-    };
-
-    gdk_gc_set_line_attributes (dock->_priv->xor_gc, 1,
-                                GDK_LINE_ON_OFF_DASH,
-                                GDK_CAP_NOT_LAST,
-                                GDK_JOIN_BEVEL);
-
-    window = gtk_widget_get_window (widget);
-
-    dash_list [0] = 1;
-    dash_list [1] = 1;
+    gint x, y;
+    GdkWindow* window = gtk_widget_get_window (GTK_WIDGET (dock));
+    gdk_window_get_origin (window, &x, &y);
     
-    gdk_gc_set_dashes (dock->_priv->xor_gc, 1, dash_list, 2);
+    if (!dock->priv->area_window) {
+        dock->priv->area_window = gdl_preview_window_new ();
+    }
 
-    gdk_draw_rectangle (window, dock->_priv->xor_gc, FALSE,
-                        rect->x, rect->y,
-                        rect->width, rect->height);
+    rect->x += x;
+    rect->y += y;
+    
+    gdl_preview_window_update (GDL_PREVIEW_WINDOW (dock->priv->area_window), rect);
+}
 
-    gdk_gc_set_dashes (dock->_priv->xor_gc, 0, dash_list, 2);
-
-    gdk_draw_rectangle (window, dock->_priv->xor_gc, FALSE,
-                        rect->x + 1, rect->y + 1,
-                        rect->width - 2, rect->height - 2);
+void
+gdl_dock_xor_rect_hide (GdlDock      *dock)
+{
+    if (dock->priv->area_window)
+        gtk_widget_hide (dock->priv->area_window);
 }
